% Program to Dynamic Specification for Michael T. Owyang and Tatevik Sekhposyan:
% "Okun's Law of the Business Cycle: Was the Great Recession All THat Different?"
% Federal Reserve Bank of St. Louis Review, October 2012




clc
clear all

[rdata, rtext] = xlsread('FinalData.xlsx','level');

data = diff(rdata(:,[1 2]));
% loose one observation to differencing
ru = data(:,1);
ry = data(:,2);

% loose one observation to differencing and two observations 
% for the two extra lags
u = ru(3:end,1);
y = [ones(size(ry,1)-2,1) ry(3:end,1) ry(2:end-1,1) ry(1:end-2) ru(2:end-1,1) ru(1:end-2)];

rdate = rdata(4:end,3);
rdatehist = rdata(4:end,4);
rdatecur = rdata(4:end,5);
rdatehisttrunc = rdata(4:end,6);
rdatejobless = rdata(4:end,7);
date = rtext(5:end,1);

% difference specification
R = 48; % size of the rolling window
t = size(u,1);
pdate = cal_ds(1949,4,t);
disp('Sample-size and observations in recessions')
[t sum(rdate)]


fullresults = nwest(u,y, floor(size(u,1)^(1/3)));
bcfullresults = nwest(u,[y rdate rdate.*y(:,2)],floor(size(u,1)^(1/3)));
bcfullresults1 = nwest(u,[y rdatehisttrunc rdatehisttrunc.*y(:,2) rdatejobless rdatejobless.*y(:,2)],floor(size(u,1)^(1/3)));
bcfullresults2 = nwest(u,[y rdatehist rdatehist.*y(:,2) rdatecur rdatecur.*y(:,2)],floor(size(u,1)^(1/3)));

% coefficients are equal to each other
rest = [0 0 0 0 0 0 1 0 -1 0;
        0 0 0 0 0 0 0 1  0 -1];
waldh1 = (rest*bcfullresults1.beta - zeros(1,1))'*(rest*bcfullresults1.vcv*rest')^(-1)*(rest*bcfullresults1.beta - zeros(1,1));
waldph1 = 1-chi2cdf(waldh1,size(rest,1));
waldh2 = (rest*bcfullresults2.beta - zeros(1,1))'*(rest*bcfullresults2.vcv*rest')^(-1)*(rest*bcfullresults2.beta - zeros(1,1));
waldph2 = 1-chi2cdf(waldh2,size(rest,1));

% coefficients are jointly zero
rest = [0 1 0 0 0 0 0 0
        0 0 1 0 0 0 0 0
        0 0 0 1 0 0 0 0];
waldh3 = (rest*bcfullresults.beta - zeros(3,1))'*(rest*bcfullresults.vcv*rest')^(-1)*(rest*bcfullresults.beta - zeros(3,1));
waldph3 = 1-chi2cdf(waldh3,size(rest,1));

disp('regression results')
disp('full-sample')
[fullresults.beta(:,1)    fullresults.se(:,1)     fullresults.beta(:,1)./fullresults.se(:,1)]
disp('full-sample with recessoin dummies')
[bcfullresults.beta(:,1)  bcfullresults.se(:,1)   bcfullresults.beta(:,1)./bcfullresults.se(:,1)]
disp('full-sample with seperate dummies - jobless recoveries')
[bcfullresults1.beta(:,1)  bcfullresults1.se(:,1)   bcfullresults1.beta(:,1)./bcfullresults1.se(:,1)]
disp('Wald test - jobless recoveries')
[waldh1 waldph1]
disp('full-sample with seperate dummies - great recession')
[bcfullresults2.beta(:,1)  bcfullresults2.se(:,1)   bcfullresults2.beta(:,1)./bcfullresults2.se(:,1)]
disp('Wald test - great recession')
[waldh2 waldph2]
disp('Wald test - coefficients are jointly zero')
[waldh3 waldph3]

disp('Andrews QLR test results')
disp('full-sample - break in the mean')
strbreak1 = QLR_hac(u,ones(t,1),y(:,2:4),y,1,1)
breakdate1 = pdate(strbreak1(3,1),:)
disp('full-sample - break in the slope')
strbreak2 = QLR_hac(u,y(:,2:4),ones(t,1),y,1,1)
breakdate2 = pdate(strbreak2(3,1),:)
disp('full-sample - break in both')
strbreak3 = QLR_hac(u,[ones(t,1) y(:,2:4)],0,y,1,1)
breakdate3 = pdate(strbreak3(3,1),:)

for i = R:t
    rollresults = nwest(u(i-R+1:i,1),y(i-R+1:i,:),floor(size(u(i-R+1:i,1),1)^(1/3)));
    rollmean(i-R+1,:) = rollresults.beta(1,1);
    rollbeta(i-R+1,:) = rollresults.beta(2,1);
    rollbeta1(i-R+1,:) = rollresults.beta(3,1);
    rollbeta2(i-R+1,:) = rollresults.beta(4,1);
end

rpdate = [1949.75+R/4-0.25:0.25:2011.75];
resdate = rdate(R:end,1);

presdate01(ismember(resdate,(0)),:) = resdate(ismember(resdate,(0)),:);
presdate01(ismember(resdate,(1)),:) = resdate(ismember(resdate,(1)),:)*min(rollmean);
presdate02(ismember(resdate,(0)),:) = resdate(ismember(resdate,(0)),:);
presdate02(ismember(resdate,(1)),:) = resdate(ismember(resdate,(1)),:)*max(rollmean);


presdate(ismember(resdate,(0)),:) = resdate(ismember(resdate,(0)),:);
presdate(ismember(resdate,(1)),:) = resdate(ismember(resdate,(1)),:)*min(rollbeta);

presdate1(ismember(resdate,(0)),:) = resdate(ismember(resdate,(0)),:);
presdate1(ismember(resdate,(1)),:) = resdate(ismember(resdate,(1)),:)*min(rollbeta1);
presdate2(ismember(resdate,(0)),:) = resdate(ismember(resdate,(0)),:);
presdate2(ismember(resdate,(1)),:) = resdate(ismember(resdate,(1)),:)*max(rollbeta1);

presdate3(ismember(resdate,(0)),:) = resdate(ismember(resdate,(0)),:);
presdate3(ismember(resdate,(1)),:) = resdate(ismember(resdate,(1)),:)*max(rollbeta2);
presdate4(ismember(resdate,(0)),:) = resdate(ismember(resdate,(0)),:);
presdate4(ismember(resdate,(1)),:) = resdate(ismember(resdate,(1)),:)*min(rollbeta2);

% Figure 6A

figure 
grayColor = [1 1 1] * 0.8; 
bar(rpdate,presdate01,'FaceColor', grayColor, 'EdgeColor','none', 'BarWidth',1)
hold on
bar(rpdate,presdate02,'FaceColor', grayColor, 'EdgeColor','none', 'BarWidth',1)
hold on
plot(rpdate,rollmean,'Linewidth',2)
hold on
plot(rpdate,fullresults.beta(1,1)*ones(t-R+1,1),'-k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(1,1)+1.96*sqrt(fullresults.vcv(1,1))*ones(t-R+1,1),'--k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(1,1)-1.96*sqrt(fullresults.vcv(1,1))*ones(t-R+1,1),'--k','Linewidth',2)
hold off
box on
legend('recession dates','rolling', 'full-sample', 'full-sample bands (95 perc.)')
title('Mean coefficient in the dynamic specification')
axis tight

% Figure 6B

figure 
grayColor = [1 1 1] * 0.8; 
bar(rpdate,presdate,'FaceColor', grayColor, 'EdgeColor','none', 'BarWidth',1)
hold on
plot(rpdate,rollbeta,'Linewidth',2)
hold on
plot(rpdate,fullresults.beta(2,1)*ones(t-R+1,1),'-k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(2,1)+1.96*sqrt(fullresults.vcv(2,2))*ones(t-R+1,1),'--k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(2,1)-1.96*sqrt(fullresults.vcv(2,2))*ones(t-R+1,1),'--k','Linewidth',2)
hold off
box on
legend('recession dates','rolling', 'full-sample', 'full-sample bands (95 perc.)')
title('Slope coefficient in the dynamic specification - lag (1)')
axis tight

% Figure 6C

figure 
grayColor = [1 1 1] * 0.8; 
bar(rpdate,presdate1,'FaceColor', grayColor, 'EdgeColor','none', 'BarWidth',1)
hold on
bar(rpdate,presdate2,'FaceColor', grayColor, 'EdgeColor','none', 'BarWidth',1)
hold on
plot(rpdate,rollbeta1,'Linewidth',2)
hold on
plot(rpdate,fullresults.beta(3,1)*ones(t-R+1,1),'-k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(3,1)+1.96*sqrt(fullresults.vcv(3,3))*ones(t-R+1,1),'--k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(3,1)-1.96*sqrt(fullresults.vcv(3,3))*ones(t-R+1,1),'--k','Linewidth',2)
hold off
box on
legend('recession dates','rolling', 'full-sample', 'full-sample bands (95 perc.)')
title('Slope coefficient in the dynamic specification - lag (2)')
axis tight

% Figure 6D


figure 
grayColor = [1 1 1] * 0.8; 
bar(rpdate,presdate3,'FaceColor', grayColor, 'EdgeColor','none', 'BarWidth',1)
hold on
bar(rpdate,presdate4,'FaceColor', grayColor, 'EdgeColor','none', 'BarWidth',1)
hold on
plot(rpdate,rollbeta2,'Linewidth',2)
hold on
plot(rpdate,fullresults.beta(4,1)*ones(t-R+1,1),'-k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(4,1)+1.96*sqrt(fullresults.vcv(4,4))*ones(t-R+1,1),'--k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(4,1)-1.96*sqrt(fullresults.vcv(4,4))*ones(t-R+1,1),'--k','Linewidth',2)
hold off
box on
legend('recession dates','rolling', 'full-sample', 'full-sample bands (95 perc.)')
title('Slope coefficient in the dynamic specification - lag (3)')
axis tight