% Program to Gap Specification for Michael T. Owyang and Tatevik Sekhposyan:
% "Okun's Law of the Business Cycle: Was the Great Recession All THat Different?"
% Federal Reserve Bank of St. Louis Review, October 2012


% Have to change which data set and graph titles are commented out in order to get results for the Gap-CBO specification


clc
clear all

[rdata, rtext] = xlsread('FinalData.xlsx','gapbandpass');
%[rdata, rtext] = xlsread('FinalData.xlsx','gapcbo');

data = rdata(:,[1 2]);
rdate = rdata(:,3);
rdatehist = rdata(:,4);
rdatecur = rdata(:,5);
rdatehisttrunc = rdata(:,6);
rdatejobless = rdata(:,7);

date = rtext(2:end,1);

u = data(:,1);
y = data(:,2);

firstrec1 = find((rdatejobless == 1) == 1);
firstrec2 = find((rdatecur == 1) == 1);
u1 = u(firstrec1(1,1):end,:);
y1 = y(firstrec1(1,1):end,:);
u2 = u(firstrec2(1,1):end,:);
y2 = y(firstrec2(1,1):end,:);


% Figure 1C; Figure 1B with other dataset

figure
scatter(y,u,40)
lsline
%hold on
%scatter(y1,u1,40,'red','*')
hold on
scatter(y2,u2,40,'red','filled')
xlabel('output gap')
ylabel('unemployment rate')
title('Gap specification - BP-filter')
% title('Gap specification - CBO')
box on

corr(y,u)

% gap specification
R = 48; % size of the rolling window
t = size(u,1);
pdate = cal_ds(1952,1,t); %this is the start date associated with BP-filter
%pdate = cal_ds(1949,1,t); %this is the start date associated with CBO
disp('Sample-size and observations in recessions')
[t sum(rdate)]

fullresults = nwest(u,[ones(t,1) y], floor(size(u,1)^(1/3)));
bcfullresults = nwest(u,[ones(t,1) y rdate rdate.*y],floor(size(u,1)^(1/3)));
bcfullresults1 = nwest(u,[ones(t,1) y rdatehisttrunc rdatehisttrunc.*y rdatejobless rdatejobless.*y],floor(size(u,1)^(1/3)));
bcfullresults2 = nwest(u,[ones(t,1) y rdatehist rdatehist.*y rdatecur rdatecur.*y],floor(size(u,1)^(1/3)));

rest = [0 0 1  0 -1 0;
        0 0 0  1  0 -1];
waldh1 = (rest*bcfullresults1.beta - zeros(1,1))'*(rest*bcfullresults1.vcv*rest')^(-1)*(rest*bcfullresults1.beta - zeros(1,1));
waldph1 = 1-chi2cdf(waldh1,size(rest,1));
waldh2 = (rest*bcfullresults2.beta - zeros(1,1))'*(rest*bcfullresults2.vcv*rest')^(-1)*(rest*bcfullresults2.beta - zeros(1,1));
waldph2 = 1-chi2cdf(waldh2,size(rest,1));


disp('regression results')
disp('full-sample')
[fullresults.beta(:,1)    fullresults.se(:,1)     fullresults.beta(:,1)./fullresults.se(:,1)]

disp('full-sample with recessoin dummies')
[bcfullresults.beta(:,1)  bcfullresults.se(:,1)   bcfullresults.beta(:,1)./bcfullresults.se(:,1)]
disp('full-sample with seperate dummies - jobless recoveries')
[bcfullresults1.beta(:,1)  bcfullresults1.se(:,1)   bcfullresults1.beta(:,1)./bcfullresults1.se(:,1)]
disp('Wald test - jobless recoveries')
[waldh1 waldph1]
disp('full-sample with seperate dummies - great recession')
[bcfullresults2.beta(:,1)  bcfullresults2.se(:,1)   bcfullresults2.beta(:,1)./bcfullresults2.se(:,1)]
disp('Wald test - great recession')
[waldh2 waldph2]


disp('Andrews QLR test results')
disp('full-sample - break in the mean')
strbreak1 = QLR_hac(u,ones(t,1),y,[ones(t,1) y],1,1)
breakdate1 = pdate(strbreak1(3,1),:)
disp('full-sample - break in the slope')
strbreak2 = QLR_hac(u,y,ones(t,1),[ones(t,1) y],1,1)
breakdate2 = pdate(strbreak2(3,1),:)
disp('full-sample - break in both')
strbreak3 = QLR_hac(u,[ones(t,1) y],0,[ones(t,1) y],1,1)
breakdate3 = pdate(strbreak3(3,1),:)


for i = R:t
    rollresults = nwest(u(i-R+1:i,1),[ones(R,1) y(i-R+1:i,1)],floor(size(u(i-R+1:i,1),1)^(1/3)));
    rollmean(i-R+1,:) = rollresults.beta(1,1);
    rollbeta(i-R+1,:) = rollresults.beta(2,1);
end
rpdate = pdate(R:end,:);
resdate = rdate(R:end,1);

presdate1(ismember(resdate,(0)),:) = resdate(ismember(resdate,(0)),:)+4;
presdate1(ismember(resdate,(1)),:) = resdate(ismember(resdate,(1)),:)*8;
presdate2(ismember(resdate,(0)),:) = resdate(ismember(resdate,(0)),:)-0.2;
presdate2(ismember(resdate,(1)),:) = resdate(ismember(resdate,(1)),:)*(-0.9);

rpdate = [1952+R/4-0.25:0.25:2008.75]; % this is associated with the BP date
%rpdate = [1949+R/4-0.25:0.25:2011.75]; % this is the date associated with the CBO measure


% Figure 5A; Figure 4A with alternate data


figure 
grayColor = [1 1 1] * 0.8; 
bar(rpdate,presdate1,'FaceColor', grayColor, 'EdgeColor','none', 'BarWidth',1)
hold on
plot(rpdate,rollmean,'Linewidth',2)
hold on
plot(rpdate,fullresults.beta(1,1)*ones(t-R+1,1),'-k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(1,1)+1.96*sqrt(fullresults.vcv(1,1))*ones(t-R+1,1),'--k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(1,1)-1.96*sqrt(fullresults.vcv(1,1))*ones(t-R+1,1),'--k','Linewidth',2)
hold off
box on
legend('recession dates','rolling', 'full-sample', 'full-sample bands (95 perc.)')
title('Mean coefficient in the gap specification')
ylim([4 8])
xlim([rpdate(1,1) rpdate(end,end)])

% Figure 5B; Figure 4B with alternate data

figure 
grayColor = [1 1 1] * 0.8; 
bar(rpdate,presdate2,'FaceColor', grayColor, 'EdgeColor','none', 'BarWidth',1)
hold on
plot(rpdate,rollbeta,'Linewidth',2)
hold on
plot(rpdate,fullresults.beta(2,1)*ones(t-R+1,1),'-k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(2,1)+1.96*sqrt(fullresults.vcv(2,2))*ones(t-R+1,1),'--k','Linewidth',2)
hold on
plot(rpdate,fullresults.beta(2,1)-1.96*sqrt(fullresults.vcv(2,2))*ones(t-R+1,1),'--k','Linewidth',2)
hold off
box on
legend('recession dates','rolling', 'full-sample', 'full-sample bands (95 perc.)')
title('Slope coefficient in the gap specification')
ylim([-0.9 -0.2])
xlim([rpdate(1,1) rpdate(end,end)])
